__author__ = 'dhamodharan.k'
import pymysql

def get_mysql_client(host_ip, username, pwd,db_name):
    connection = pymysql.connect(host=host_ip,user=username,
                                 db=db_name,
                                 password=pwd,
                                 autocommit=True,
                                 cursorclass=pymysql.cursors.DictCursor,
                                 charset='utf8',
                                 max_allowed_packet=16*1024)
    return connection


def _get_last_inserted(connection_obj):
    with connection_obj.cursor() as ms_cursor:
        ins_id_query = 'SELECT LAST_INSERT_ID();'
        ins_id = query_actions(connection_obj, ins_id_query, 'select')
    ms_cursor.close()
    return ins_id

def _get_affected_rows(connection_obj):
    with connection_obj.cursor() as ms_cursor:
        ins_id_query = 'SELECT ROW_COUNT();'
        ins_id = query_actions(connection_obj, ins_id_query, 'select')
    ms_cursor.close()
    return ins_id

def query_actions(connection_obj, query, query_type):
    result = None
    with connection_obj.cursor() as cursor:
        cursor.execute(query)
        if query_type == 'select':
            result = cursor.fetchall()
        if query_type == 'insert':
            result = _get_last_inserted(connection_obj)[0]['LAST_INSERT_ID()']
        if query_type == 'update':
            result = str(_get_affected_rows(connection_obj)[0]['ROW_COUNT()']) + 'row(s) affected '
    cursor.close()
    return result



__author__ = 'dhamodharan.k'
import pyodbc
from collections import OrderedDict
import warnings


def get_sql_client(server_ip,database,userName,pwd):
    """
    :param server_ip: 195.16.40.82
    :param database: SampleDB
    :param userName: root
    :param pwd: root
    :return: connection object
    """
    cnxn = pyodbc.connect('DRIVER={SQL Server};SERVER='+server_ip+';DATABASE='+database+';UID='+userName+';PWD='+pwd)
    cursor = cnxn.cursor()
    return cursor

def query_actions(db_object, query, query_type):
    """

    :param db_object: connection object
    :param query: query
    :param query_type: query type. either update/insert/select
    :return: query result
    """
    query = query.replace("'NULL'",'NULL')
    if query_type == 'insert' or query_type == 'update':
        db_object.execute(query)
        db_object.commit()
    if query_type == 'select':
        db_object.execute(query)
        rows = db_object.fetchall()
        return rows

def insert_records(connection_obj, table_data:dict, data_dict: dict, unique_columns:list):
    """
    :param connection_obj: connection object
    :param table_data: {"db_name":"sampleDb","table_name":"stu"}
    :param data_dict: update/insert data as key value pair
    :param unique_columns: unique only columns as list
    :return: scope_id
    """
    db = table_data.get('db_name')
    table_name = table_data.get('table_name')
    query = "select column_name FROM "+str(db)+".INFORMATION_SCHEMA.COLUMNS WHERE TABLE_NAME = N'" + table_name + "'"
    rows = query_actions(connection_obj, query, 'select')
    rows = [i[0] for i in rows][1:]
    final_dict = OrderedDict({i: (str(data_dict[i]).replace("'",'"')if i in list(data_dict.keys()) else 'NULL') for i in rows})
    values = list(final_dict.values())
    column = list(final_dict.keys())
    if len(unique_columns) != 0:
        unique_columns = [i + "='" + final_dict[i] + "'" for i in unique_columns]
        unique_columns = ' AND '.join(unique_columns) if len(unique_columns) > 1 else unique_columns[0]
        data_check = "select * from " + table_name + " where " + unique_columns
        data_exists = query_actions(connection_obj, data_check, 'select')
    else: data_exists = []
    if len(data_exists) == 0:
        insert_query = "INSERT INTO " + table_name + " (" + ", ".join(column) + ") VALUES ('" + "', '".join( values) + "')"
        query_actions(connection_obj, insert_query, 'insert')
        scope_id_query = 'SELECT SCOPE_IDENTITY()'
        scope_id_query_result = query_actions(connection_obj, scope_id_query, 'select')
        scope_id = scope_id_query_result[0][0]
    else:
        warnings.warn('Data already Exist in given table')
        scope_id = data_exists[0][0]
    return scope_id



__author__ = 'dhamodharan.k'
import re
import ftfy
import xml.etree.ElementTree as ET
from fuzzywuzzy import fuzz
import inspect
import collections
from user_agent import generate_user_agent
from dateutil import parser
from os import path
import os
from bs4 import BeautifulSoup
import csv
import uuid
from datetime import datetime
import shutil
import hashlib
import glob


def get_clean_text(string:str):
    """
    :param string: takes raw string
    :return: returns clean string and tab,space,newlines,encoding issues fixed

    """
    string = ftfy.fix_text(string)
    # string = re.sub('\s+', ' ', string)
    # string = re.sub('\n|\t|^\s+\|\s+$', '', string)
    # string = string.strip()
    return string

def get_clean_html(html_chunk):
    '''
    Method to clean the given HTML Chunk and returns the cleaned version of HTML
    :param soup: raw HTML chunk
    :return: Cleaned HTML Chunk

    '''
    soup = str(html_chunk)
    clean_data = re.sub("(<img.*?>)", "", soup, 0, re.IGNORECASE | re.DOTALL | re.MULTILINE)
    clean_data = re.sub("(<input.*?>)", "", clean_data, 0, re.IGNORECASE | re.DOTALL | re.MULTILINE)
    clean_data = re.sub('(href=".*?")', "", clean_data, 0, re.IGNORECASE | re.DOTALL | re.MULTILINE)
    clean_data = re.sub('(<script .*?</script>)', "", clean_data, 0, re.IGNORECASE | re.DOTALL | re.MULTILINE)
    clean_data = re.sub('(<!--.*?-->)', "", clean_data, 0, re.IGNORECASE | re.DOTALL | re.MULTILINE)
    clean_string = str(clean_data).replace('\n', '').replace('\t', '').replace('\r', '').replace('\xa0',' ').strip()
    clean_string = clean_string.replace('Click here', '').replace('Read More', '')
    clean_string = clean_string.replace('<button', '<div').replace('</button>', '')
    clean_string = ' '.join(clean_string.split())
    clean_string = ftfy.fix_text(clean_string)
    clean_string = ftfy.fix_text_encoding(clean_string)
    soup = BeautifulSoup(clean_string, 'lxml')
    for tags in soup.find_all():
        if not tags.text:
            tags.decompose()
    clean_string = str(soup)
    return clean_string

def get_string_from_html(html_chunk):
    """
    :param html_chunk: Takes HTML chunks
    :return: removes hyperlinks and tags from string for text analysis process
    """
    clean_string = re.sub("(<.*?>)", "", str(html_chunk), 0, re.IGNORECASE | re.DOTALL | re.MULTILINE)
    clean_string = re.sub("(<style.*?</style>)", "", str(clean_string), 0, re.IGNORECASE | re.DOTALL | re.MULTILINE)
    clean_string = get_clean_text(clean_string)
    return  clean_string

def get_numbers_from_string(string:str):
    """
    :param string: takes a string as input
    :return: returns numbers from given string
    """
    num = ''.join([i for i in string if i.isnumeric()])
    return num

def get_alpha_from_string(string:str):
    """
    :param string: takes are string as input
    :return: returns only alpha chars from given text
    """
    except_list = list('/\."$*<>:|?')
    string = ''.join([i for i in string if i.isalpha() or i not in except_list])
    return string

def compare_two_strings(string_one,string_two):
    """
    :param string_one: any text string
    :param string_two: any text string
    :return: returns match ratio in integer using fuzzy logic
    """
    return fuzz.ratio(string_one,string_two)

############################## LIST Operations ##############################

def join_elements(source_list:list, separator:str=' '):
    """
    :param source_list: Takes list as input
    :param separator: any special chars or any printable chars
    :return: join given list elements with given separator and returns a string
    """
    return separator.join(source_list)

def get_clean_list(list_x:list):
    '''clean the list values using get_clean_text()'''
    return [get_clean_text(i) for i in list_x]

def compare_two_list(first:list, second:list):
    '''Compare first list with second and returns the difference'''
    second = set(second)
    return [item for item in first if item not in second]


def find_duplicates_from_list(list:list):
    '''Returns a duplicate values in list'''
    return [item for item, count in collections.Counter(list).items() if count > 1]

def get_unique_values(sequence:list):
    '''
    Get unique list values without loosing the order
    :param sequence: python list [1,4,7,9,0,1,4,6,7]
    :return: returns python unique ordered list [1,4,7,9,0,6,7]
    '''
    seen = set()
    return [x for x in sequence if not (x in seen or seen.add(x))]


############################## Dict Operations ##############################


def get_clean_dict(dict:dict):
    '''passed through get_clean_text() for both keys and values'''
    for key, value in dict.items():
        dict[get_clean_text(str(key)) if type(key) == str else key] = get_clean_text(str(value)) if type(value) == str else value
    return dict


def modify_jsondata(abbreviations_dict:dict,target_dict:dict):
    '''{'name':'dharan'} to {'person_name':'dharan'}. here abbreviation dict has {'person_name':'name'}'''
    collected_data = {}
    for source_attribute, target_attribute in abbreviations_dict.items():
        if target_attribute:
            if target_attribute in target_dict:collected_data[source_attribute] = target_dict.get(target_attribute, '')
            else:collected_data[source_attribute] = ''
        else: collected_data[source_attribute] = ''
    collected_data = {key: value for key, value in collected_data.items() if str(key) != str(value)}
    return collected_data


def combine_dict(dict1, dict2):
    '''
    Merge dictionaries and keep values of common keys in list

    :param dict1: {'fruit':'apple','name':'dharan'}

    :param dict2: {'fruit':'Orange'}

    :return:  {'fruit':['apple','Orange'],'name':'dharan'}

    '''
    dict3 = {**dict1, **dict2}
    for key, value in dict3.items():
        if key in dict1 and key in dict2:
            dict3[key] = [value, dict1[key]]
    return dict3

############################## Support Operations ##############################

def get_current_user_name():
    try:
        try:
            import pwd
            user_name = pwd.getpwuid(os.geteuid()).pw_name
        except ImportError:
            import getpass
            user_name = getpass.getuser()
    except: user_name = None
    return user_name

def get_current_platform():
    '''
    A method to find the current working operating system/platform

    :return: any one of linux/os x/windows
    '''
    from sys import platform
    if platform == "linux" or platform == "linux2":
        return "linux"
    elif platform == "darwin":
        return "os x"
    elif platform == "win32":
        return "windows"

def get_class_variables(class_obj):
    '''Simple method to get the list of variable from the given class object'''
    variables = list(class_obj.__dict__.keys())
    return variables

def get_current_script_name():
    """
    Simple method used to get the calling function name.
    :return: returns callers script name without file extension
    """
    frame = inspect.stack()[1]
    module = inspect.getmodule(frame[0])
    filename = module.__file__
    filename = str(filename).split("/")[-1].split('.')[0]
    return filename

def get_user_agent(**kwargs):
    domain_specific = {}
    user_agent = 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/66.0.3359.117 Safari/537.36'
    try:
        domain_name = kwargs.get('domain_name','NA')
        user_agent = domain_specific.get(domain_name) if domain_name in domain_specific.keys() else generate_user_agent(device_type='desktop',os=['win','linux','mac'])
    except: pass
    return user_agent

def generate_fileName(project, ext):
    time_ = datetime.now().isoformat(sep="_").split(".")[0].replace(':', '.')
    if project:
        fileName = "export_{}_{}.{}".format(project, time_, ext)
    else:
        fileName = "export_{}.{}".format(time_, ext)
    return fileName

def generate_uuid_by_digits(digits):
    """
    Returns the unique identifier for given number of digits

    :param digits: int value holds number of digits needs to be returned

    :return:
    """
    return uuid.uuid4().hex[:int(digits)].upper()

def genearate_hash_key(string):
    '''
    Generate the Hash string out of given string

    :param string: Any string value

    :return: md5 hexdigest string
    '''
    h = hashlib.md5(str(string).encode())
    return h.hexdigest()


############################## File Operations ##############################

def get_base64Image(encoded_data,path_to_store,file_name):
    try:
        import base64
        if not os.path.exists(path_to_store):
            os.makedirs(path_to_store)
        imgdata = base64.b64decode(encoded_data)
        with open(path_to_store + file_name + '.jpg', 'wb') as f:
            f.write(imgdata)
    except Exception as error:
        return error

def get_filename(file_name):
    '''Used to get file name from the path'''
    temp = str(file_name).lower().split("\\")[-1].split('.')
    if len(temp)>=2:
        return "{}.{}".format(temp[0],temp[1])
    else: return None

def move_to(source,destination,filename):
    '''Used to move the file from source to destination'''
    shutil.move(source, destination + "\\" + filename)

def copy_to(source,destination,filename):
    '''Used to copy the file from source to destination'''
    shutil.copy(source, destination + "\\" + filename)

def list_files(diretory,ext:str=''):
    '''List all the files in the directory'''
    if ext: return glob.glob(diretory + "\\*." + str(ext))
    else: return glob.glob(diretory + "\\*.*")

def read_text_file(file_name):
    try:
        with open(file_name,'r', encoding='utf8') as fileReadObj:
            rows = fileReadObj.read()
            return rows
    except:
        with open(file_name,'r', encoding='ISO-8859-1') as fileReadObj:
            rows = fileReadObj.read()
            return rows

def read_xml_file(path,file):
    parser = ET.XMLParser(encoding='utf-8')
    mytree = ET.parse(path + file, parser=parser)
    myroot = mytree.getroot()
    return mytree, myroot

def read_xml_file_as_bs4(path,file_name):
    with open(os.path.join(path, file_name), 'r', encoding='utf-8') as file_handler:
        soup = BeautifulSoup(file_handler.read(), 'xml')
    return soup

def write_text_file(filepath, filename, content):
    filepath = os.getcwd() if filepath == '' else filepath
    with open(filepath + "/" + filename, 'a', encoding='utf8') as fileWriteObj:
        fileWriteObj.write(str(content) + "\n")

def write_csv_file(filepath,filename,content:dict,headers:list):
    filepath = os.getcwd() if filepath == '' else filepath
    with open(filepath + "/" + filename, 'a', encoding='utf8') as fileWriteObj:
        writer_ptr = csv.DictWriter(fileWriteObj, delimiter='\t',fieldnames=headers)
        writer_ptr.writerow(content)

def check_file_exists(file_dir):
    '''
    Simple method to check the existence of the given file.

    :param file_dir: file path

    :return: Bool value
    '''
    return path.exists(file_dir)

def get_sizeof(num, suffix='o'):
    """Readable file size
    :param num: Bytes value
    :type num: int
    :param suffix: Unit suffix (optionnal) default = o
    :type suffix: str
    :rtype: str
    """
    for unit in ['', 'k', 'M', 'G', 'T', 'P', 'E', 'Z']:
        if abs(num) < 1024.0:
            return "%3.1f %s%s" % (num, unit, suffix)
        num /= 1024.0
    return "%.1f%s%s" % (num, 'Yi', suffix)


################## Date Operations #######################


def get_current_date(format="ISO"):
    if format== "ISO":
        return datetime.now().isoformat()
    else:
        return str(datetime.now().strftime("%d_%m_%Y"))

def get_yearFromDate(self, date_string):
    '''
    The method used to get year from the given date string
    :param date_string: any date string
    :return: int year from given date string
    '''
    year = 0
    try:
        t = parser.parse(date_string)
        year = t.year
    except:
        pass
    return int(year)

def convert_date_to(input_string, format):
    '''
    A method used to convert the date string in given format
    :param input_string: Date String
    :param format: format name which date string need to be converted.Currently supports,
        iso : converts date into iso date format
    :return: converted date in required format
    '''
    converted = input_string
    t = parser.parse(input_string)
    if format == 'iso': converted = str(t.isoformat())
    return converted

########################### Email functions ####################################

def send_email_with_attachment(projectName, userMessage, attachment, recipients,default_email_recipients=''):
    """
    A email method used to send the attachments with given information
    :param projectName: Project Name should mentioned in the subject
    :param userMessage: Email Body
    :param attachment: Attachment file path
    :param recipients: email recipients separated by comma
    :return: None
    """
    if attachment:
        today_date = str(datetime.now().strftime("%d/%m/%Y"))
        Subject = "Automated Alert {}:{}".format(projectName, today_date)
        message = "'Hi All,\n\n " + userMessage + "\n\n Thanks, \n MrSnippets'"
        if recipients:
            os.system("echo " + message + " | mail -s '" + str(Subject) + "' -c " + str(default_email_recipients) + " -a " + attachment + " " + recipients)
        else:
            os.system("echo " + message + " | mail -s '" + str(Subject) + "'" + " -a " + attachment + " " + default_email_recipients)

def email_notification(notificationType, project, message, recipients,default_email_recipients=''):
    """
    Method used to send the email using linux's mail method with user given values
    :param notificationType: Type of the notification. which will be mentioned in the subject
    Planed Options,
        1. INFO
        2. ERROR
        3. ALERT
        4. WARNING
    Example : <TYPE> Notification: <PROJECT> --> INFO Notification : AIMS
    :param project: Project Name
    :param message: Mail Body
    :return: None
    """
    subject = "{} Notification : {}".format(notificationType, project)
    message = "'Hi All,\n\n " + message + ".\n\n Thanks, \n MrSnippets'"
    os.system("echo " + message + " | mail -s '" + str(subject) + "' -c " + str(default_email_recipients) + " " + recipients)


__author__ = 'dhamodharan.k'
import requests
import json
from MrSnippets.data_mining import extract_domain_name_from_url
from MrSnippets.utilities import *
from bs4 import BeautifulSoup
from itertools import cycle
from selenium import webdriver

def UPDATE_FETCHER_LOG(logDict):
    """
    the idea behind this is to store the logs in database.
    :param logDict: dict data
    :return: None
    """
    write_text_file('','request.log',str(logDict))

proxy_sets = {"xx.xx.xx.xxx:80", "xx.xx.xx.xxx:90"}
proxy_pool = cycle(proxy_sets)
def get_proxy():
    """
    :return: proxy dict
    """
    proxy = next(proxy_pool)
    return {"http": proxy, "https": proxy}

def get_selenium_response(url, timeout):
    """
    A method to get the selenium based response for the given URL
    :param url:  Input URL
    :param timeout: Max wait time for the request
    :return: BeautifulSoup object
    """
    browser = None
    if get_current_platform() == "windows":
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument('-headless')
        browser = webdriver.Chrome("chromedriver.exe", chrome_options=chrome_options)
    # else:
        # os.environ["PATH"] += os.pathsep + BaseDir.fireFoxPath
        # options = webdriver.FirefoxOptions()
        # options.add_argument('-headless')
        # browser = webdriver.Firefox(executable_path=BaseDir.geckodriverPath, firefox_options=options,timeout=timeout)
    browser.get(url)
    html = browser.page_source
    soup = BeautifulSoup(html, "html.parser")
    browser.close()
    browser.quit()
    return soup

def get_response(url, response_type, attempt=0, **kwargs):

    """
    A method used to get the web response and returns in user defined format.

    :param url: URL to get web response

    :param response_type: One of Pre-defined method to return the fetched response

        soup : Returns the web response as BeautifulSoup object

        text : Returns the web response as raw text

        json : returns the response as json object

        content : returns the web response as it is. Use full in fetching file objects

        response : returns the response code

    :param attempt: holds the number of retry should be performed

    :param kwargs: Holds multiple options to utilize the same method in multiple ways.

        data : Used in post method to provide data

        params : Used to pass parameters data for post method

        time_out : Used to set the timeout for the requests and its an integer

        verify : Verify the HTTP Certificate. Will be Bool value. By default True

        method : pre-defined value either POST or GET

        headers : dict data holds the HTTP headers. By default holds the User-Agent String

        allow_redirects : Bool value

        stream : Bool value

        request_type : Pre-defined value. Either requests or selenium

        dom_parser : method to parse HTML in BeautifulSoup. By default html5lib

    :return: returns as per response_type args


    """
    RESPONSE_DATA = None
    response = None
    data = kwargs.get('data', {})
    gateway = kwargs.get('gateway', 'requests')
    params = kwargs.get('params', {})
    timeout = kwargs.get('time_out', 60)
    verify = kwargs.get('verify', True)
    method = kwargs.get('method', None)
    domain = kwargs.get('domain', '')
    if not domain : domain = extract_domain_name_from_url(url)
    headers = kwargs.get('headers',{})
    allow_redirects = kwargs.get('allow_redirects', True)
    proxy =  kwargs.get('proxy', True)
    stream =  kwargs.get('stream', False)
    dom_parser = kwargs.get('dom_parser','html5lib')
    headers.update({'User-Agent': get_user_agent(domain_name=domain)})
    proxy = get_proxy() if proxy else {}
    try: currentIP = str(proxy['http']).split("@")[1]
    except: currentIP = ''
    if currentIP: print("[{}] : {}".format(currentIP,url))
    else: print("[{}] : {}".format("NO PROXY*",url))

    hashData = hashlib.md5(str(url).encode())
    URLMETA = {'IP':str(currentIP),"domain": domain,'rowkey':str(hashData.hexdigest()),"url": url,'timeStamp': str(datetime.now()),'LogType':'NA','User-Agent': headers.get('User-Agent','NA'),'Method': str(method).upper() if method else "GET",'Error':'NA'}

    REQUEST_META_LOG = {"ResponseType":response_type,'RequestData':json.dumps(kwargs)}
    URLMETA.update({"RequestMeta":REQUEST_META_LOG})
    try:
        if gateway == 'selenium':
            try:
                response = get_selenium_response(url, timeout)
            except Exception as error:
                response = None
                URLMETA.update({'log_type': 'Error', 'response_code': 'NA', 'retry': attempt, 'Error': str(error)})
        elif gateway == "requests":
            if str(method).upper() == 'POST':
                response = requests.post(url, data = data, params=params, headers=headers, proxies = proxy, timeout=timeout, verify=verify)
                response_status = response.status_code
                URLMETA.update({'log_type':'Request','response_code': str(response_status), 'retry': attempt})
            else:
                response = requests.get(url, data=data, params=params, headers=headers, proxies = proxy, timeout=timeout, verify=verify, allow_redirects=allow_redirects, stream=stream)
                response_status = response.status_code
                URLMETA.update({'log_type': 'Request','response_code': str(response_status), 'retry': attempt})
    except Exception as error:
        response = None
        if 'Caused by ProxyError' in str(error):
            URLMETA.update({'log_type': 'Error', 'response_code': 'NA', 'retry': attempt, 'Error': str(error)})
            print('Attempting with different Proxy! Proxy ' + proxy['http'] + ' has been identified by server!')
        else:
            print('Error occurred while fetching page response. Error Message -- ' + str(error) + str(url))
    if response:
        try:
            if response.status_code == 200 and gateway != "selenium":
                page_response = response
                if response_type == 'soup': RESPONSE_DATA = BeautifulSoup(page_response.text, dom_parser)
                elif response_type == 'text': RESPONSE_DATA = page_response.text
                elif response_type == 'content': RESPONSE_DATA = response.content
                elif response_type == 'json': RESPONSE_DATA = response.json()
                elif response_type == 'response': RESPONSE_DATA = response
            if response.status_code == 429 or response.status_code == 401 and gateway != "selenium":
                get_response(url, response_type, attempt + 1, **kwargs)
            elif gateway == "selenium":
                RESPONSE_DATA = response
            else:
                response_status = response.status_code
                URLMETA.update({'log_type': 'Error','response_code': str(response_status), 'retry': attempt})
        except Exception as e:
            URLMETA.update({'log_type': 'Error','response_code': 'NA', 'retry': attempt,'Error':str(e)})
    else:
        if attempt < 6:
            print('Attempt {attempt} : Empty response found for '.format(attempt=attempt) + currentIP + " ! Attempting with different Proxy..")
            get_response(url, response_type, attempt + 1, **kwargs)
    UPDATE_FETCHER_LOG(URLMETA)
    return RESPONSE_DATA


def download_file(url,dir,file_name,extension):
    """

    :param url: download link
    :param dir: directory to store the file
    :param file_name: file name for the download, Optional None
    :param extension: file extension without dot
    :return: None
    """

    if not os.path.exists(dir): os.makedirs(dir)
    try:
        response_obj = get_response(url,'response', proxy=True, stream=True)
        if response_obj:
            if not file_name:
                file_name = genearate_hash_key(str(url))
            with open(dir + file_name + "." + extension, 'wb') as f:
                for chunk in response_obj.iter_content(1024):
                    f.write(chunk)
            return True
    except Exception as ValueError:
        print(ValueError)


__author__ = 'dhamodharan.k'
import nltk
from geotext import GeoText
from nltk.corpus import stopwords
from nltk.stem.wordnet import WordNetLemmatizer
from sklearn.feature_extraction.text import TfidfVectorizer
from nltk.tokenize import word_tokenize
from nltk.tokenize import sent_tokenize
import nltk.classify.util
from nltk.sentiment.vader import SentimentIntensityAnalyzer
from nltk.stem import PorterStemmer
from MrSnippets import utilities as util
from nltk.tokenize import SExprTokenizer
import string
import spacy

nlp = spacy.load("en_core_web_sm")

try:
    stop_words = set(stopwords.words('english'))
except:
    raise FileNotFoundError("Stopwords not Found! use \n import nltk\nnltk.download('stopwords')\n in the python console")

# Use Python console to download stopwords
# import nltk
# nltk.download('stopwords')

# Support functions

def _remove_stopwords(word_tokens:list):
    """
    :param word_tokens:
    :return:
    """
    filtered_sentence = [w for w in word_tokens if not w in stop_words]
    return filtered_sentence

def _normalize_text(text:str):
    """
    :param text:
    :return:
    """
    remove_punctuation_map = dict((ord(char), None) for char in string.punctuation)
    updated_text = text.lower()
    updated_text = util.get_clean_text(updated_text)
    updated_text = updated_text.translate(remove_punctuation_map)
    tokens = get_tokenized(updated_text,True,'word')
    return get_stemming(tokens)

def _create_frequency_table(text_string) -> dict:
    """
    we create a dictionary for the word frequency table.
    For this, we should only use the words that are not part of the stopWords array.
    Removing stop words and making frequency table
    Stemmer - an algorithm to bring words to its root word.
    :rtype: dict
    """
    stopWords = set(stopwords.words("english"))
    words = word_tokenize(text_string)
    ps = PorterStemmer()
    freqTable = dict()
    for word in words:
        word = ps.stem(word)
        if word in stopWords:
            continue
        if word in freqTable:
            freqTable[word] += 1
        else:
            freqTable[word] = 1
    return freqTable

def _score_sentences(sentences, freqTable) -> dict:
    """
    score a sentence by its words
    Basic algorithm: adding the frequency of every non-stop word in a sentence divided by total no of words in a sentence.
    :rtype: dict
    """
    sentenceValue = dict()
    for sentence in sentences:
        word_count_in_sentence = (len(word_tokenize(sentence)))
        word_count_in_sentence_except_stop_words = 0
        for wordValue in freqTable:
            if wordValue in sentence.lower():
                word_count_in_sentence_except_stop_words += 1
                if sentence[:10] in sentenceValue:
                    sentenceValue[sentence[:10]] += freqTable[wordValue]
                else:
                    sentenceValue[sentence[:10]] = freqTable[wordValue]

        if sentence[:10] in sentenceValue:
            sentenceValue[sentence[:10]] = sentenceValue[sentence[:10]] / word_count_in_sentence_except_stop_words
    return sentenceValue

def _find_average_score(sentenceValue) -> float:
    """
    Find the average score from the sentence value dictionary
    :rtype: int
    """
    sumValues = 0
    for entry in sentenceValue:
        sumValues += sentenceValue[entry]
    average = (sumValues / len(sentenceValue))
    return average

def _generate_summary(sentences:str, sentenceValue, threshold):
    sentence_count = 0
    summary = ''
    for sentence in sentences:
        if sentence[:10] in sentenceValue and sentenceValue[sentence[:10]] >= (threshold):
            summary += " " + sentence
            sentence_count += 1
    return summary

# Basic NLP Operations
def get_tokenized(sentence: str, ignore_stopwords: bool = False,use:str='SExpr'):
    """
    :param sentence:
    :param ignore_stopwords:
    :param use:
    :return:
    """
    word_tokens = []
    if use=="word": word_tokens = word_tokenize(sentence)
    if use=="SExpr":
        tk = SExprTokenizer()
        word_tokens = tk.tokenize(sentence)
    if ignore_stopwords:
        filtered_sentence = _remove_stopwords(word_tokens)
        return filtered_sentence
    else:
        return word_tokens

def sentence_tokenization(raw_text:str):
    """
    :param raw_text:
    :return:
    """
    return sent_tokenize(raw_text)

def get_stemming(word_tokens:list):
    """
    Stemming is the process of reducing the words(generally modified or derived) to their word stem or root form.
    The objective of stemming is to reduce related words to the same stem even if the stem is not a dictionary word.

    :param word_tokens: unprocced word tokens
    :return: processed list
    """
    ps = PorterStemmer()
    return [ps.stem(w) for w in word_tokens]

def get_lemmatize_data(word_tokens: list):
    """
    Lemmatisation is the process of reducing a group of words into their lemma or dictionary form
    :param word_tokens:
    :return:
    """
    lem = WordNetLemmatizer()
    return [lem.lemmatize(w) for w in word_tokens]

def get_standardize_words(tokens: list, lookup_dict: dict):
    """
    :param tokens:
    :param lookup_dict:
    :return:
    """
    new_words = []
    for word in tokens:
        if word.lower() in lookup_dict:
            word = lookup_dict[word.lower()]
        new_words.append(word)
    return new_words

def generate_ngrams(tokens: list, n):
    """
    :param tokens:
    :param n:
    :return:
    """
    output = []
    for i in range(len(tokens) - n + 1):
        output.append(tokens[i:i + n])
    return output

def generate_pos_tags(sentence, ignore_stopwords: bool = False):
    """
    :param sentence:
    :param ignore_stopwords:
    :return:
    """
    tokens = get_tokenized(sentence, ignore_stopwords)
    pos_tagged = nltk.pos_tag(tokens)
    return pos_tagged

def generate_named_entity(sentence):
    """
    :param sentence:
    :return:
    """
    doc = nlp(sentence)
    data = []
    for ent in doc.ents:
        dict_data = {}
        dict_data['TEXT'] = ent.text
        dict_data['START'] = ent.start_char
        dict_data['END'] = ent.end_char
        dict_data['LABEL'] = ent.label_
        data.append(dict_data)
    return data

def sentiment_analysis(sentence,algorithm:str="VADER"):
    """
    :param sentence:
    :param algorithm:
    :return:
    """
    sid = SentimentIntensityAnalyzer()
    sentiment_result = sid.polarity_scores(sentence)
    return sentiment_result

def get_cosine_similarity(text1:str, text2:str):
    """
    :param text1:
    :param text2:
    :return:
    """
    vectorizer = TfidfVectorizer(tokenizer=_normalize_text, stop_words='english')
    tfidf = vectorizer.fit_transform([text1, text2])
    return ((tfidf * tfidf.T).A)[0,1]

def extract_geo_data(text:str,required='country'):
    """
    :param text:
    :param required:
    :return:
    """
    data = []
    if required == "countries":
        data = GeoText(text).countries
    if required == "cities":
        data = GeoText(text).cities
    if required == "nationalities":
        data = GeoText(text).nationalities
    return data

def generate_text_summary(text:str):
    freq_table = _create_frequency_table(text)
    sentences = sent_tokenize(text)
    sentence_scores = _score_sentences(sentences, freq_table)
    threshold = _find_average_score(sentence_scores)
    summary = _generate_summary(sentences, sentence_scores, 1.3 * threshold)
    return summary


__author__ = 'dhamodharan.k'
from MrSnippets.helpers import *
import dateparser
import pycountry
from bs4 import BeautifulSoup
from geopy.geocoders import Nominatim
geolocator = Nominatim(user_agent="MrSnippets")
from dateutil import parser
from urllib.parse import urlparse
import tldextract


############################## HTML Functions ##############################

def get_element(soup, tag="div", attributeName='class', attributeValue='profile'):
    """
    :param soup: HTML Bs4 object
    :param tag: tag needs to be extracted. Example Div
    :param attributeName: variable name. Example class
    :param attributeValue: example "profile" . class="profile"
    :return: subset of html element
    """
    try:
        html_tag = soup.find(tag, {attributeName: attributeValue})
    except:
        return None
    return html_tag

def get_elements(soup, tag="div", attributeName='class', attributeValue='profiles'):
    """
    :param soup:
    :param tag:
    :param attributeName:
    :param attributeValue:
    :return:
    """
    try:
        html_tags = soup.findAll(tag, {attributeName: attributeValue})
    except:
        return None
    return html_tags

def get_element_by_tag(soup,selector_string:str):
    """

    :param soup:
    :param selector_string:
    :return:
    """
    try:
        element = BeautifulSoup(selector_string,'html5lib').body.next
        return soup.find(element.name,element.attrs)
    except: return None

def get_elements_by_tag(soup,selector_string:str):
    """

    :param soup:
    :param selector_string:
    :return:
    """
    try:
        element = BeautifulSoup(selector_string,'html5lib').body.next
        return soup.find_all(element.name,element.attrs)
    except: return None

def get_sibling_text(soup, child: str, sibling: str, contains_string: str, sibling_type="prev|next"):
    """

    :param soup:
    :param child:
    :param sibling:
    :param contains_string:
    :param sibling_type:
    :return:
    """
    result = soup.find(child, string=contains_string)
    if sibling_type == 'next':
        text = result.findNext(sibling).text
    elif sibling_type == 'prev':
        text = result.findPrevious(sibling).text
    else:
        text = None
    return text

############################## Semi automated extration methods ##############################

def extract_hyper_link(soup_chunk, patterns: list, **kwargs):
    """
    Extracting the profile url based on patterns found of profile url & returns list of dicts
    :param soup_chunk:
    :param patterns:
    :param kwargs:
    :return:
    """
    assert type(patterns) == list, "List require in Patterns"
    prefix = kwargs.get('prefix', '')
    with_text = kwargs.get('with_text', False)
    validate = lambda url: [1 for i in patterns if str(url).lower().__contains__(str(i).lower())]
    if with_text:
        if prefix: urls_found = [{"text":get_clean_text(link.text),"link":"{}{}".format(prefix,link["href"])} for link in soup_chunk.find_all("a", href=True) if len(validate(link["href"])) > 0]
        else: urls_found = [{"text":get_clean_text(link.text),"link":link["href"]} for link in soup_chunk.find_all("a", href=True) if len(validate(link["href"])) > 0]
    else:
        if prefix: urls_found = ["{}{}".format(prefix,link["href"]) for link in soup_chunk.find_all("a", href=True) if len(validate(link["href"])) > 0]
        else: urls_found = [link["href"] for link in soup_chunk.find_all("a", href=True) if len(validate(link["href"])) > 0]
    if urls_found: return urls_found
    else:
        if with_text: return [{'link':'','text':''}]
        else: return []

def extract_vcard_link(soup_chunk, **kwargs):
    """

    :param soup_chunk:
    :param kwargs:
    :return:
    """
    patterns = kwargs.get('patterns', ['.vcf', 'vcard'])
    prefix = kwargs.get('prefix', '')
    validate = lambda url: [1 for i in patterns if str(url).lower().__contains__(str(i).lower())]
    if prefix:
        urls_found = ["{}{}".format(prefix, link["href"]) for link in soup_chunk.find_all("a", href=True) if len(validate(link["href"])) > 0]
    else:
        urls_found = [link["href"] for link in soup_chunk.find_all("a", href=True) if len(validate(link["href"])) > 0]
    return urls_found

def extract_image_link(soup_chunk, **kwargs):
    """

    :param soup_chunk:
    :param kwargs:
    :return:
    """
    patterns = kwargs.get('patterns', ['media', 'images', 'image'])
    prefix = kwargs.get('prefix', '')
    validate = lambda url: [1 for i in patterns if str(url).lower().__contains__(str(i).lower())]
    if prefix:
        urls_found = ["{}{}".format(prefix, link["src"]) for link in soup_chunk.find_all("img", src=True) if len(validate(link["src"])) > 0]
    else:
        urls_found = [link["src"] for link in soup_chunk.find_all("img", src=True) if len(validate(link["src"])) > 0]
    return urls_found

def extract_vcard_data(vcard_text: str):
    """

    :param vcard_text:
    :return:
    """
    extracted = {}
    try:
        contents = vcard_text.splitlines()
        tel = [i for i in contents if 'TEL;WORK;VOICE' in str(i)]
        if not tel: tel = [i for i in contents if 'TEL;type=WORK,voice:' in str(i)]
        extracted['Tel'] = tel[0].split(':')[-1] if tel else ''
        email = [i for i in contents if 'EMAIL;PREF;INTERNET' in str(i)]
        if not email:
            email = [i for i in contents if 'EMAIL;TYPE=internet,pref' in str(i)]
        elif not email:
            email = [i for i in contents if 'EMAIL;type=INTERNET,pref' in str(i)]
        extracted['Email'] = email[0].split(':')[-1] if email else ''
        address = [i for i in contents if 'ADR' in str(i)]
        extracted['Location'] = address[0].split(';')[-4]
        return extracted
    except:
        return {}

def extract_email_addresses(string):
    """

    :param string:
    :return:
    """
    string = str(string)
    string = string.replace(' (at) ', '@')
    string = string.replace('(at)', '@')
    string = string.replace(' <at> ', '@')
    string = string.replace('<at>', '@')
    r = re.compile(r'[\w\-][\w\-\.]+[@]+[\w\-][\w\-\.]+[a-zA-Z]{1,4}')
    return r.findall(string)

def extract_phone_numbers(html_chunk):
    """

    :param html_chunk:
    :return:
    """
    number = None
    try:
        html_chunk = re.sub(r'&nbsp',':',str(html_chunk))
        pattern = re.compile( r'(\+?\d+(?:[- \\)]+\d+)+)')
        match = pattern.findall(html_chunk)
        match = [re.sub(r'[,.]', '', el) for el in match if len(re.sub(r'[()\-.,\s+]', '', el)) > 6]
        match = [re.sub(r'\D$', '', el).strip() for el in match]
        match = [el for el in match if len(re.sub(r'\D', '', el)) <= 15]
        try:
            for el in list(match):
                if len(el.split('-')) > 3 or len(el.split(' ')) > 3: continue
                for x in el.split("-"):
                    try:
                        if x.strip()[-4:].isdigit():
                            if int(x.strip()[-4:]) in range(1900, 2100):
                                match.remove(el)
                    except:pass
        except:pass
        number = match
    except: pass
    return number

def extract_social_links(html_source):
    """

    :param self:
    :param html_source:
    :return:
    """
    patterns = ['facebook','twitter','linkedin','tumblr','instagram','skype','pinterest','youtube','flickr']
    validate = lambda url: [1 for i in patterns if str(url).lower().__contains__(str(i).lower())]
    urls_found = [link["href"] for link in html_source.find_all("a", href=True) if len(validate(link["href"])) > 0]
    return urls_found

def extract_meta_data(html_source):
    """

    :param self:
    :param html_source:
    :return:
    """
    meta_chunks = html_source.find_all('meta')
    valid_meta = ['description', 'keywords']
    meta_data,meta = {"DG": 'meta'},[]
    for meta_tags in meta_chunks:
        content = meta_tags.get('content', '')
        name = meta_tags.get('name', '')
        if name in valid_meta: meta_data[name] = content
    meta.append(meta_data)
    return meta

def extract_date_from_string(date_string: str):
    '''
    A method used to extract the date from any given date string.
    :param date_string: A string having date information
    :return: returns the extracted date in YYYY-MM-DD format
    '''
    correct_date = None
    try:
        t = dateparser.parse(date_string)
        correct_date = t.strftime("%Y-%m-%d")
    except:
        pass
    if not correct_date:
        try:
            t = parser.parse(date_string)
            correct_date = t.strftime("%Y-%m-%d")
        except:
            pass
    if not correct_date:
        try:
            t = parser.parse(date_string, fuzzy_with_tokens=True)
            if t: correct_date = t[0].strftime("%Y-%m-%d")
        except:
            pass
    return correct_date


def extract_countries_from_text(input_text):
    '''
    A method used to find the country mentions in given string
    :param input_text: Any text
    :return: country name in list
    '''
    countries = [country.name for country in pycountry.countries if
                 str(input_text).lower().__contains__(str(country.name).lower())]
    countries = countries if countries else None
    if not countries:
        location = geolocator.geocode(str(input_text))
        address = location.address
        countries = [country.name for country in pycountry.countries if
                     str(address).lower().__contains__(str(country.name).lower())]
    return countries

def extract_url_prefix(url_string: str):
    '''
    Returns the URL prefix value as string
    :param url_string: input url
    :return: URL prefix
    '''
    m = re.search('https?://([A-Za-z_0-9.-]+).*', str(url_string))
    prefix = str(url_string).split("/")[0] + "//" + m.group(1)
    return prefix

def extract_domain_name_from_url(url):
    if "http" in str(url) or "www" in str(url):
        parsed = tldextract.extract(url)
        parsed = ".".join([i for i in parsed if i])
        return parsed
    else: return "NA"

############################## Support Functions ##############################

def get_parsed_url(url_string):
    return urlparse(url_string)

def validate_url(url):
    '''
    A method to validate the given URL string for URL schema

    :param url: input url string

    :return: returns bool value if conditions are met
    '''
    regex = re.compile(
        r'^(?:http|ftp)s?://'
        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'
        r'localhost|'
        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
        r'(?::\d+)?'
        r'(?:/?|[/?]\S+)$', re.IGNORECASE)
    return re.match(regex, str(url))

def fix_url_format(self, url, prefix):
    """

    A method used to fix the URL related issues in the data extraction process. As of now it will support following formation issues.

        "/pipeline/archives/2020/04/22/the-politics-of-hydroxychloroquine",
        "pipeline/archives/2020/04/22/the-politics-of-hydroxychloroquine",
        "http://google.com/pipeline/archives/2020/04/22/the-politics-of-hydroxychloroquine",
        "https://google.com/pipeline/archives/2020/04/22/the-politics-of-hydroxychloroquine",
        "//www.google.com/pipeline/archives/2020/04/22/the-politics-of-hydroxychloroquine",
        "www.google.com/pipeline/archives/2020/04/22/the-politics-of-hydroxychloroquine",
        "www.google.com/pipeline/archives/2020/04/22/the-politics-of-hydroxychloroquine",

    :param url: String input

    :param prefix: Base URL string like https://google.com

    :return: updated URL
    """

    fixed_url = url
    url = url[1:] if str(url).startswith("/") else url
    url = url[1:] if str(url).startswith("/") else url
    domain_name = self.extract_domain_name(prefix)
    url_schema = "https" if str(prefix).__contains__('https') else "http"
    if not self.validate_url(url):
        url = "{}/{}".format(domain_name, url) if not str(url).__contains__(domain_name) else url
        if url.startswith('http://www.'):
            fixed_url = 'http://' + url[len('http://www.'):]
        if url.startswith('https://www.'):
            fixed_url = 'https://' + url[len('https://www.'):]
        if url.startswith('www.'):
            fixed_url = '{}://'.format(url_schema) + url[len('www.'):]
        if not url.startswith("http"):
            fixed_url = "http://{}".format(url)
    return fixed_url

############################## Data Manipulations ##############################

def get_standard_name(name:str):
    """
    :param name: takes name strings
    :return: returns the name in standard format
    """
    reobj = re.compile(r'(?sm)((Dr|Ph\.D|Jr|LL\.M|Avv|\b[A-Z]{1}|\b[a-z]{1}|Prof)\.|\b(Dr|Jr|Ph\.D|LL\.M|PhD|LLM|Dipl.-Jur.|Jur|DAPI|\*|,\s*[A-Z]{2,4}|[A-Z]{1})(\s|$)|\(.*?\)|(".*?")|(/|;).*?$)')
    match = reobj.search(name)
    if match:
        refined_name = re.sub(r'(?sm)((Dr|Ph\.D|Jr|LL\.M|\b[A-Z]{1}|\b[a-z]{1}|Prof)\.|\b(Dr|Jr|Ph\.D|LL\.M|PhD|LLM|Dipl.-Jur.|Jur|DAPI|\*|,\s*[A-Z]{2,4}|[A-Z]{1})(\s|$)|\(.*?\)|(".*?")|(/|;).*?$)', '', name)
        refined_name = refined_name.replace('  ', ' ').strip()
    else: refined_name = get_clean_text(name)
    return refined_name

def split_name(name_string:str,reverse_it:bool=False,**kwargs):
    """
    :param name_string: takes the name as string
    :param reverse_it: accepts bool value. 1, revers the last name, first name else first name, last name
    :param kwargs: delimiter, delimiter value between first name, last name
    :return:
    """
    split_by = kwargs.get('delimiter'," ")
    assert len(name_string.split(split_by)) > 1, "Given String is not splittable by given delimiter key"
    nameData = {}
    name = ftfy.fix_text(refine_name(name_string))
    name_split = str(name).split(split_by)
    if reverse_it:
        name_split = list(reversed(name_split))
        name = " ".join(name_split)
    fname,lname = name_split[0],name_split[-1]
    nameData['FullName'] = name
    nameData['FirstName'] = fname
    nameData['LastName'] = lname
    return nameData


__author__ = 'Dhamodharan Karuppuswamy'
import os
import pickle

current_path = os.getcwd()

def get_city_list():
    try:
        with open(current_path + "\\data\\cities.pickle",'rb') as f:
            cities_data = pickle.load(f)
            return list(cities_data.City)
    except: return []

def get_state_list():
    try:
        with open(current_path + "\\data\\cities.pickle",'rb') as f:
            cities_data = pickle.load(f)
            return list(set(cities_data.State))
    except: return []

def get_city_info_obj():
    try:
        with open(current_path + "\\data\\cities.pickle", 'rb') as f:
            cities_data = pickle.load(f)
            return cities_data
    except: return None





__author__ = 'dhamodharan.k'
import xml.etree.ElementTree as ET
import ast
from xml.etree.ElementTree import Element, SubElement
from xml.sax.saxutils import escape

def pandas_to_xml(**kwargs):
    """
    A method used to convert pandas to solr xml
    :param kwargs: A keyword args mainly contains,
        dataFrame : pandas data frame object
        fileName : output file name
        list_variables : Values should considered as list
        xml_declaration : True/False by default True
    :return: he file will be stored in file directory
    """
    _IGNORE_ME = ['nan', '', 'NA', 'NaT', 'None', '[]']
    df = kwargs.get("dataFrame")
    fileName = kwargs.get("fileName")
    xml_declaration = kwargs.get("xml_declaration", True)
    file_path = kwargs.get("file_path", './')
    tree = ET.ElementTree()
    rootElement = Element('add')
    records = df.to_dict('rows')
    for dictRow in records:
        subDoc = SubElement(rootElement, 'doc')
        for key, value in dictRow.items():
            if str(value) not in _IGNORE_ME:
                if isinstance(value,list):
                    for textData in value:
                        if str(textData) not in _IGNORE_ME:
                            textData = str(textData)
                            current_group = SubElement(subDoc, "field", {'name': key})
                            current_group.text = escape(textData)
                elif isinstance(value,str):
                    value = str(value)
                    current_group = SubElement(subDoc, "field", {'name': key})
                    current_group.text = escape(value)
    tree._setroot(rootElement)
    tree.write(file_path + fileName, encoding='utf8', xml_declaration=xml_declaration, method='xml')

def create_xml_string_from_dict(data_dict:dict):
    xml_string = '<doc>\n'
    f_close = '</field>\n'
    for each_key in data_dict:
        if isinstance(data_dict[each_key], list) and len(data_dict[each_key]) > 0:  # this is for list members
            for element in data_dict[each_key]: # over elements of list members
                if element is not None:
                    for each_inner_key in element:  # over the keys of inner dict
                        if element[each_inner_key] is not None:
                            if element[each_inner_key]:
                                current_line = '<field name="' + each_inner_key + '">' + escape(
                                    element[each_inner_key]) + f_close
                                xml_string = xml_string + current_line
        if (not isinstance(data_dict[each_key], list)) and data_dict[each_key] is not None:
            current_line = '<field name="' + each_key + '">' + escape(str(data_dict[each_key])) + f_close
            xml_string = xml_string + current_line
    xml_string = xml_string + '</doc>\n'
    return xml_string


__author__ = 'dhamodharan.k'
from pymongo import MongoClient

MONGO_CLIENT_IP = 'xx.xx.xx.xxx:27017'
MONGO_CLIENT = MongoClient(MONGO_CLIENT_IP)

def get_mongo_client(database_name, collection_name):
    """
    :param database_name: database name in string
    :param collection_name: collection name in string
    :return: mongo cursor object
    """
    db = MONGO_CLIENT[database_name]
    db_result = db[collection_name]
    return db_result

def list_db():
    """
    :return:
    """
    db_list = MONGO_CLIENT.list_database_names()
    return db_list

def list_collections(db:str):
    """
    :param db: database name
    :return:
    """
    _db = MONGO_CLIENT[db]
    return _db.list_collection_names()

def get_summarize(db:str,collection:str):
    """

    :param db: database name
    :param collection: collection name
    :return: returns dict with total no of records in given collection
    """
    connection = get_mongo_client(db,collection)
    out_dict = {}
    out_dict['DatabaseName'] = db
    out_dict['CollectionName'] = collection
    out_dict['TotalRecords'] = connection.count_documents({})
    return out_dict

def get_sample(db:str,collection:str,query_by:str,value:str,limit:int=1):
    """
    :param db: db name
    :param collection: collection name
    :param query_by: key
    :param value: value
    :param limit: int value and defult is 1
    :return: returns list of values
    """
    connection = get_mongo_client(db,collection)
    if str(query_by).lower().__contains__('id'):
        record = [i for i in connection.find({query_by:int(value)},{'_id':0},limit=limit)]
    else:
        record = [i for i in connection.find({query_by: value}, {'_id': 0}, limit=limit)]
    return record

def update_record(connection:dict,query_by:str,query_by_value,data:dict):
    """

    :param connection: collection name
    :param query_by: key
    :param query_by_value: value
    :param data: dict records
    :return: None
    """
    assert ('db' and 'collection' in list(connection.keys())), 'Required attributes not found!'
    try:
        connection = get_mongo_client(connection['db'], connection['collection'])
        if connection.find_one({query_by: query_by_value}):
            connection.update_one({query_by: query_by_value},{'$set':data})
            return True
        else : return False
    except Exception as error: raise Exception(error)

def update_attribute(connection:dict,query_by:str,query_by_value,data:dict,attributes:list):
    """
    :param connection: connection object
    :param query_by: query by key
    :param query_by_value: value
    :param data: data dict
    :param attributes: list of multi valued variable's
    :return: None
    """
    connection = get_mongo_client(connection['db'], connection['collection'])
    exist_check = connection.find_one({query_by: query_by_value})
    if exist_check:
        _id = exist_check['_id']
        for update_key in attributes:
            old_value = exist_check.get(update_key,[])
            current_value = data.get(update_key,[])
            if type(old_value) == list and type(current_value) == list:
                new_without_Dup = set(old_value + current_value)
                connection.update_one({'_id': _id}, {'$set': {update_key: list(new_without_Dup)}})
    else: connection.insert_one(data)

def create_index(connection:dict,index_attributes:list,ascending:bool=True):
    """
    :param connection: connection object
    :param index_attributes: list of variable names
    :param ascending: bool value
    :return: None
    """
    assert ('db' and 'collection' in list(connection.keys())),'Required attributes not found!'
    try:
        connection = get_mongo_client(connection['db'],connection['collection'])
        index_dict = {i: 1 if ascending else -1 for i in index_attributes}
        connection.create_index(index_dict)
    except Exception as error:
        return error


import io
import os
import sys
from shutil import rmtree
from setuptools import find_packages, setup, Command

NAME = "MrSnippets"
DESCRIPTION = "A complete collection of commonly used code Snippets in Python"
URL = "https://github.com/dhamodharanrk/MrSnippets"
EMAIL = "dhamodharanrk@gmail.com"
AUTHOR = "Dhamodharan Karuppuswamy"

REQUIRED = ["beautifulsoup4>=4.3.3","requests>=2.18.4","html5lib>=1.0b10","user_agent>=0.1.9","selenium>= 3.141.0","PyMySQL>=0.9.3","pymongo>=3.8.0",
            "ftfy>=5.5.1","tldextract>=2.2.1","bleach>=3.1.0","python-csv>=0.0.11","nltk>=3.4.5","spacy>=2.3.2","sklearn>=0.0",
            "pytest-shutil>=1.6.0","Pillow>=2.2.1","pyodbc>=4.0.26","fuzzywuzzy>=0.18.0","pycountry>=19.8","geopy>=1.21","python-dateutil>=2.8.1",
            "urllib3>=1.25","tldextract>=2.2.2","geotext>=0.4.0","dateparser>=1.0.0","uuid>=1.30","glob2>=0.7"
            ]

HERE = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(HERE, "README.md"), encoding="utf-8") as f:
    long_description = "\n" + f.read()

about = {}
with open(os.path.join(HERE, NAME, "__version__.py")) as f:
    exec(f.read(), about)

class UploadCommand(Command):

    description = "Build and publish the package."
    user_options = []
    @staticmethod
    def status(s):
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        try:
            self.status("Removing previous builds…")
            rmtree(os.path.join(HERE, "dist"))
        except OSError:
            pass
        self.status("Building Source and Wheel (universal) distribution…")
        setup_str = "{0} setup.py sdist bdist_wheel --universal"
        os.system(setup_str.format(sys.executable))
        self.status("Uploading the package to PyPi via Twine…")
        os.system("twine upload dist/*")
        sys.exit()
setup(
    name=NAME,
    version=about["__version__"],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    url=URL,
    packages=find_packages(),
    install_requires=REQUIRED,
    include_package_data=True,
    license="BSD 2-Clause",
    classifiers=[
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8"
    ],
    cmdclass={"upload": UploadCommand},
)


from PIL import Image
from PIL import ImageEnhance

def adjust_brightness(input_image, output_image, factor):
    """
    :param input_image:
    :param output_image:
    :param factor:
    :return:
    """
    image = Image.open(input_image)
    enhancer_object = ImageEnhance.Brightness(image)
    out = enhancer_object.enhance(factor)
    out.save(output_image)

def adjust_contrast(input_image, output_image, factor):
    """

    :param input_image:
    :param output_image:
    :param factor:
    :return:
    """
    image = Image.open(input_image)
    enhancer_object = ImageEnhance.Contrast(image)
    out = enhancer_object.enhance(factor)
    out.save(output_image)

def adjust_sharpness(input_image, output_image, factor):
    """

    :param input_image:
    :param output_image:
    :param factor:
    :return:
    """
    image = Image.open(input_image)
    enhancer_object = ImageEnhance.Sharpness(image)
    out = enhancer_object.enhance(factor)
    out.save(output_image)

def resize_image(input_image_path,output_image_path,size):
    """

    :param input_image_path:
    :param output_image_path:
    :param size:
    :return:
    """
    original_image = Image.open(input_image_path)
    resized_image = original_image.resize(size)
    resized_image.save(output_image_path)

def scale_image(input_image_path,output_image_path,width=None,height=None):
    """

    :param input_image_path:
    :param output_image_path:
    :param width:
    :param height:
    :return:
    """
    original_image = Image.open(input_image_path)
    w, h = original_image.size
    if width and height: max_size = (width, height)
    elif width: max_size = (width, h)
    elif height: max_size = (w, height)
    else: raise RuntimeError('Width or height required!')
    original_image.thumbnail(max_size, Image.ANTIALIAS)
    original_image.save(output_image_path)

def black_and_white(input_image_path,output_image_path):
   """
   :param input_image_path:
   :param output_image_path:
   :return:
   """
   color_image = Image.open(input_image_path)
   bw = color_image.convert('L')
   bw.save(output_image_path)

def rotateImage(image_path, degrees_to_rotate, saved_location):
    """

    :param image_path:
    :param degrees_to_rotate:
    :param saved_location:
    :return:
    """
    image_obj = Image.open(image_path)
    rotated_image = image_obj.rotate(degrees_to_rotate)
    rotated_image.save(saved_location)

def flipImage(image_path, saved_location,direction):
    """

    :param image_path:
    :param saved_location:
    :param direction:
    :return:
    """
    image_obj = Image.open(image_path)
    if direction=="FLIP_LEFT_RIGHT":
        rotated_image = image_obj.transpose(Image.FLIP_LEFT_RIGHT)
        rotated_image.save(saved_location)
    elif direction=="FLIP_TOP_BOTTOM":
        rotated_image = image_obj.transpose(Image.FLIP_TOP_BOTTOM)
        rotated_image.save(saved_location)
    elif direction=="TRANSPOSE":
        rotated_image = image_obj.transpose(Image.TRANSPOSE)
        rotated_image.save(saved_location)

def cropImage(image_path, coords, saved_location):
    """
    :param image_path:
    :param coords:
    :param saved_location:
    :return:
    """
    '''crop('image.jpg', (161, 166, 706, 1050), 'cropped.jpg')'''
    image_obj = Image.open(image_path)
    cropped_image = image_obj.crop(coords)
    cropped_image.save(saved_location)